#include "ParticleSystem.h"
#include <windows.h>
#include <fstream>

float SpawnRateCheck = 0;

Vec multiply(Vec v, float scalar)
{
	Vec l_v;
	l_v.x = v.x * scalar;
	l_v.y = v.y * scalar;
	l_v.z = v.z * scalar;
	return l_v;
}

Vec mapPlaneToSphere(float x, float y)
{
	float z = -1.0f + 2.0f * x;
	float phi = 2.0f * 3.14159f * y;
	float theta = asin(z);
	Vec temp;
		temp.x = cos(theta) * cos(phi),
		temp.y = cos(theta) * sin(phi),
		temp.z = z;
	return temp;
}

float Length(Vec v)
{
	return sqrtf((v.x * v.x) + (v.y * v.y) + (v.z * v.z));
}

Vec Normalize(Vec v)
{
	Vec l;
	float l_Length = Length(v);
	if (l_Length != 0)
	{
		l.x = v.x / l_Length;
		l.y = v.y / l_Length;
		l.z = v.z / l_Length;
	}
	return l;
}

Vec subtract(Vec v, Vec v2)
{
	Vec l_v;
	l_v.x = v.x - v2.x;
	l_v.y = v.y - v2.y;
	l_v.z = v.z - v2.z;

	return l_v;
}
Vec add(Vec v, Vec v2)
{
	Vec l_v;
	l_v.x = v.x + v2.x;
	l_v.y = v.y + v2.y;
	l_v.z = v.z + v2.z;
	return l_v;
}
ParticleSystem::ParticleSystem()
{
}

ParticleSystem::~ParticleSystem()
{
	delete[] m_ParticleBuff;
}

void ParticleSystem::Update(Particle a_arr[], float dt)
{	
	SpawnRateCheck -= dt;
	for (unsigned int i = 0; i < m_MaxSize; i++)
	{
		if(SpawnRateCheck < 0)
		{
			spawn(a_arr);
			SpawnRateCheck += SpawnRate;
		}
		else if (a_arr[i].active)
		{	
			
			a_arr[i].LifeTime -= dt;
			a_arr[i].Vel = add(a_arr[i].Vel, multiply(a_arr[i].Accel,dt));
			a_arr[i].Vel = multiply(a_arr[i].Vel, 1.0f);//drag
			a_arr[i].Pos = add(a_arr[i].Pos, a_arr[i].Vel);
			
			if (a_arr[i].LifeTime < 0)
			{
				deSpawn(a_arr[i]);	
			}
		}
		
	}
}

void ParticleSystem::spawn(Particle a_arr[])
{
	for (unsigned int i = 0; i < m_MaxSize; i++)
	{
		if (!a_arr[i].active)
		{
			
			float randNum = (rand() / 32767.0f) * 6.28318530718; //num between 0 and 2 * pi

			Vec VeloVec;
			VeloVec.x = sin(randNum) * 10.0f * ((rand() / 32767.0f) * 2.0f);
			VeloVec.y = ((rand() / 32767.0f) * 20.0f);
			VeloVec.z = cos(randNum) * 10.0f * ((rand() / 32767.0f) * 2.0f);

			VeloVec.x = VeloVec.x / 100.f;
			VeloVec.y = VeloVec.y / 100.f;
			VeloVec.z = VeloVec.z / 100.f;

			a_arr[i].Pos.x = m_Pos.x;; //+ PosVec.x;
			a_arr[i].Pos.y = m_Pos.y;; //+ PosVec.y;
			a_arr[i].Pos.z = m_Pos.z;; //+ PosVec.z;

			a_arr[i].Vel.x = VeloVec.x;
			a_arr[i].Vel.y = VeloVec.y;
			a_arr[i].Vel.z = VeloVec.z;

			a_arr[i].Accel.x = -m_Grav.x;
			a_arr[i].Accel.y = -m_Grav.y;
			a_arr[i].Accel.z = -m_Grav.z;

			a_arr[i].LifeTime = lifeTime;
			a_arr[i].active = true;
			i = m_MaxSize;
		}
	}
}
void ParticleSystem::deSpawn(Particle &part)
{
	part.active = false;
}